
//Global variables and tables

//List of stages whose skyboxes and weather we probably shouldn't be touching
rawset(_G, "xmasBlacklist", 
	{"egg rock", "black core", "azure temple", //Vanilla
     "midnight freeze", "sunken plant", "inner sanctum", "prismatic angel", "spatial void", "primordial abyss", "mystic realm", "dimension warp", //Mystic Realm
     "space walk"} //Tortured Planet
)
rawset(_G, "xmasBl", false)		//Bool checking if a christmas stage is blacklisted from the above list. Used by the HUD.
rawset(_G, "emblems", {})		//Stage goal emblems
rawset(_G, "checkpoints", {})	//Checkpoints
rawset(_G, "active", 1)			//How many emblems are left in this stage
rawset(_G, "mapSpawn", 0)		//Where player 1 should spawn in our current map
rawset(_G, "activateAll", true) //Should all previous checkpoints be activated when activating a new one?
rawset(_G, "KRIMA", 0)			//Was the christmas command used?
rawset(_G, "emDifficulty", 2)   //What emblems should be created from
rawset(_G, "superEH", false)    //Whether or not maps will start with all 7 emeralds

//function run on level load to reset things and place emblems
rawset(_G, "initialize", function()
	if not (gametype == GT_EMBHUNT) return end
	allReset()
	local id = 1
	if superEH then emeralds = 127
	else emeralds = 0 end

	//Iterating through things to find relevant objects
	for em in mapthings.iterate do
		//Finding vanilla emblems or emerald tokens and adding them to our table
		if (em.type == mobjinfo[MT_EMBLEM].doomednum and (emDifficulty == 2 or emDifficulty == 3))
		or (em.type == mobjinfo[MT_TOKEN].doomednum and (emDifficulty == 1 or emDifficulty == 3))
			if (not R_PointInSubsectorOrNil(em.x*FRACUNIT, em.y*FRACUNIT)) or (em.x == 32767 and em.y == 32767) continue end //We found a emblem out of bounds! Skip this one...
			local goal = P_SpawnMobj(em.x*FRACUNIT, em.y*FRACUNIT, 0, MT_GOALEMBLEM)
			if P_CheckDeathPitCollide(goal) then goal.z = P_FloorzAtPos(em.x*FRACUNIT, em.y*FRACUNIT, em.z*FRACUNIT, 16*FRACUNIT) + FRACUNIT end
			goal.color = 64
			goal.shadowscale = FRACUNIT
			goal.zdelay = 2 //solution to Z issue taken from shine's Adventure Emblems
			goal.zdest = em.z*FRACUNIT
			goal.id = id
			table.insert(emblems, goal)
			id = $+1
		end
		//Adding P1's spawn position as the checkpoint warp's warp position
		if em.type == 1 then mapSpawn = em end
	end
	//If no emblem was found, place an emblem at the stage end sign
	local activateSpecial = false
	if not #emblems
		for sign in mapthings.iterate do
			if sign.type == mobjinfo[MT_SIGN].doomednum
				if activateSpecial then continue end
				local goal = P_SpawnMobj(sign.x*FRACUNIT, sign.y*FRACUNIT, 0, MT_GOALEMBLEM)
				goal.zdelay = 2
				goal.zdest = sign.z
				goal.color = 64
				goal.shadowscale = FRACUNIT
				goal.signEmblem = true //marking our emblem as a special one. if any of these is approached, the level ends.
				table.insert(emblems, goal)
				activateSpecial = true
			end
		end
	end

	if activateSpecial 
		active = -1
	elseif not #emblems 
		active = 0 
	else 
		active = #emblems 
	end

	//IT'S CHRISTMAAAAAS! Let's make a rotating 3D present!
	if KRIMA and #emblems
		for i=1,#emblems do giftify(emblems[i]) end
	end

	//Christmas weather, night sky and color tint! Let's check our blacklist before applying them.
	if KRIMA and KRIMA >= 2
		xmasBl = false
		for i=0,#xmasBlacklist
			if string.lower(mapheaderinfo[gamemap].lvlttl) == xmasBlacklist[i] then xmasBl = true end
		end
		if not xmasBl
			P_SetupLevelSky(6)
			P_SetSkyboxMobj(nil)
			P_SwitchWeather(2)
		end
	end

	//Finding star posts for the warp system, then sorting them by order
	for mo in mobjs.iterate() do
		if mo.info == mobjinfo[MT_STARPOST] table.insert(checkpoints, mo) end
	end
	table.sort(checkpoints, angleCheck)

	//Give each valid player a sorted list of checkpoints
	//Invalid players can get checkpoints{} later in their playerThink when they become valid
	for p in players.iterate
		if not p.valid continue end
		p.warpTable = {}
		for i=1,#checkpoints do table.insert(p.warpTable, false) end
	end
end)

//3D distance used by getProximity
rawset(_G, "dist3D", function(mo1,mo2)
	local x = mo2.x - mo1.x
	local y = mo2.y - mo1.y
	local z = mo2.z - mo1.z
	return FixedHypot(FixedHypot(x,y),z)
end)

//Proximity checker for the emblem radar
rawset(_G, "getProximity", function(mo, target)
	if not mo.valid or not target.valid return 1 end
	local dist = dist3D(mo,target)/FRACUNIT
	if target.inactive return 1 end
	//Data taken from source code
	local i = 1
	if dist < 128
		i = 6
	elseif dist < 512
		i = 5
	elseif dist < 1024
		i = 4
	elseif dist < 2048
		i = 3
	elseif dist < 3072
		i = 2
	end
	return i
end)

//Angle checker for sorting checkpoints
rawset(_G, "angleCheck", function(a,b)
	return (a.spawnpoint.angle + a.spawnpoint.extrainfo*360) < (b.spawnpoint.angle + b.spawnpoint.extrainfo*360)
end)

//Convert strings to tables
rawset(_G, "stringToTable", function(str)
	local t = {}
	for i = 1, #str do t[i] = str:sub(i, i) end
	return t
end)

//Reset everything
rawset(_G, "allReset", function(a,b)
	mapSpawn = 0
	for _,emb in ipairs(emblems)
		if emb and emb.valid then P_RemoveMobj(emb) end
	end
	emblems = {}
	checkpoints = {}
	active = 1
	for p in players.iterate do
		p.warpTable = nil
		p.warpDest = nil
		p.checkWarp = nil
		p.warpMenu = false
		p.flashFade = false
		p.warpTimer = -1
	end
end)

//Function for creating 3D presents around mobjs
rawset(_G, "giftify", function(emblem)
	emblem.flags2 = $ | MF2_DONTDRAW
	emblem.gift = {}
	local giftColor = P_RandomRange(1,#skincolors-1)
	for i=1, 4
		//Four sides of our gift
		local giftSide = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGSIDE)
		giftSide.angle = ANGLE_90*i
		giftSide.myemblem = emblem
		giftSide.scale = FRACUNIT/2
		giftSide.color = giftColor
		table.insert(emblem.gift, giftSide)
		if i>2
			//Top and bottom of our gift
			local giftTop = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGTOP)
			giftTop.myemblem = emblem
			giftTop.renderflags = $ | RF_NOSPLATBILLBOARD
			giftTop.spriteyoffset = -30*FRACUNIT
			giftTop.top = i==3 and true or false
			giftTop.state = i==3 and S_GITO or S_GIBO
			giftTop.scale = FRACUNIT/2
			giftTop.color = giftColor
			table.insert(emblem.gift, giftTop)
		end
		if i==4
			//And finally a little ribbon
			local ribbon = P_SpawnMobjFromMobj(emblem, 0, 0, 0, MT_EMGRIB)
			ribbon.myemblem = emblem
			ribbon.scale = 3*FRACUNIT/5
			ribbon.color = giftColor//ColorOpposite(giftColor)
			ribbon.shadowscale = 2*FRACUNIT
			table.insert(emblem.gift, ribbon)
		end
	end
end)